/*!
  \file   batwing.c
  \brief  The main file for the console batwing project

  A program to get posterior trees for genealogies
 
  \author Ian Wilson
  \date   2004-06-09
*/
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "modeltime.h"
#include "node.h"
#include "tree.h"
#include "random.h"
#include "nrutil.h"
#include "metro.h"
#include "lhood.h"
#include "cutjoin.h"
#include "pars.h"
#include "cutil.h"
#include "newick.h"
#ifdef CHECK
#include "check.h"
#endif
#include "missing.h"
#include "myio.h"

#ifdef FORENSIC
#include "forensic.h"
forensic match;
#endif

/**************************************************************************/
/*  The Main Function  - Batwing (infile) (outfile) (seed)                */
/**************************************************************************/
int main(int argc, char *argv[])
{
  int i,j,pic_count=1;
  char picname[40],picstr[30],*inputfile="",*outputfile="",*seed="";
  const char *message;
  tree t;
  parameters p;  
  FILE *OUTFILE, *OUTTIME, *PARS, *CONSENSUSP,*CONSENSUST;
  time_t start_time, end_time;
  double time_diff;
  
  time(&start_time);

  #ifdef FORENSIC
    double match_prob_est, match_prob_ci;  
  #endif
	
  if (argc==1) {
    fprintf(stderr,"no arguments entered:\n");
    inputfile=getstringfromkeyboard("input file","infile");
  } else {
    inputfile=argv[1];
    if (argc>=3) outputfile=argv[2];
    if (argc==4) seed=argv[3];
  }
  
  if (argc>4) {
    fprintf(stderr,"use:\n%s infile (outfile) (seed)\n",argv[0]);  
    exit(-1);
  }

  p=setup_parameters(inputfile,outputfile,seed,quiet); 

  strcpy(picname,p.outfilename);
  strcpy(picstr,p.outfilename);
  /*
  if ((message=check_parsb(&p,quiet))!=NULL) {
    fprintf(stderr,"%s\n",message);	exit(1);
  }
  */

  if (p.initialfilename!=NULL) {

    t=read_tree(p.initialfilename,p.constsites);
    if (p.seed!=-1) {
      fprintf(stderr,"new seed set to %d\n",p.seed);
      starTup(p.seed);	
    } else {
      fprintf(stderr,"starting from old random seed\n");
    }
    PARS = openstemleafoutputfile(picname,"par");
  } else {
    read_parameters_data(&p); 
    t = partreestartup(&p);
    printpardetails(&p,stdout,1);
    PARS = openstemleafoutputfile(picname,"par");
    printpardetails(&p,PARS,0);	
  } 

  fprintf(stderr,".");
  fprintf(stderr,"starting up\n");
  OUTFILE=openoutputfile(picname);
  #ifdef FORENSIC_VERBOSE
    output_names(OUTFILE,&t,&p,&match);
  #else
    output_names(OUTFILE,&t,&p,NULL);
  #endif
  strcat(picname,".iit");
  printalltreeinfo(picname,&t,&p);
  if (p.tconsensus)  CONSENSUST=openstemleafoutputfile(picstr,"tcns");
  if (p.pconsensus)  CONSENSUSP=openstemleafoutputfile(picstr,"pcns");
#ifdef FORENSIC
  if (t.missing.n == 0) {
    nrerror("In forensic mode, exactly one missing must be specified. Please place it in the second line in the datafile.");
  }
  match=setupforensic(&t.sample[2],t.sample[1].STRgeno,t.nstr,&t.mut);
#endif 
#ifdef CHECK
  checktree(&t,"before start");
#endif
/* New - warm up -> */
  if (p.warmup <= 0) {
    fprintf(stderr,"no warming up set -- going directly to actual run\n");
  } else {
    fprintf(stderr,"warming up\n");
    
    for (i=1;i<=p.warmup;i++) {
      for (j=1;j<=p.Nbetsamp;j++) {
        metro_step(&t,p.treebetN,p.move_nodes);	
      }
      
      if (i%20==0) {
        if (i%1000==0) {
          fprintf(stderr,".%d\n",i); 
        } else {
          fprintf(stderr,"."); 
        }
        fflush(stderr);
      }
    }
    
    fprintf(stderr,"\n%d samples made to warm up - now the actual run\n", p.warmup);  
  }

  /* Checking that forensic containers haven't been touched yet */
  if (match.attempts != 0.0 || t.missing_matches_suspect != 0 || t.missing_doesnt_match_suspect != 0) {
    fprintf(stdout,"ERROR: forensic containers was touched during warm up!\n");
    exit(999);
  }
  
/* <- New - warm up */
  for (i=1;i<=p.reps;i++) {
    for (j=1;j<=p.Nbetsamp;j++) {
      metro_step(&t,p.treebetN,p.move_nodes);	

      #ifdef FORENSIC
        missing_matches_suspect(&t);
        checkmatches(&match,t.growth.N.x,t.root->time,t.totallength);
      #endif
    }
    #ifdef FORENSIC_VERBOSE
      output_line(OUTFILE,&t,&p,&match);
    #else
      output_line(OUTFILE,&t,&p,NULL);
    #endif

    if (p.pconsensus) write_shapeNewickpoptree(CONSENSUSP,&t.populationtree);
    if (p.tconsensus) write_Newickshape(t.root,t.sample,CONSENSUST);
		
    if (i%20==0 ) {
      if (i%1000==0) {
        fprintf(stderr,".%d\n",i); 
        
        #ifdef FORENSIC
        if (i > 50000 && p.adaptive_matches_stop > 0 && p.adaptive_matches_stop < t.missing_matches_suspect) {
          /*
          2.00 * sd => 95.45 % CI
          1.96 * sd => 95.00 % CI
          1.00 * sd => 68.27 % CI
          */
          match_prob_ci = sqrt(match.overall_avg_M2 / (match.overall_avg_n - 1));
          
          if (match.calc_prob > match.overall_avg_mean - match_prob_ci && match.calc_prob < match.overall_avg_mean + match_prob_ci) {
            fprintf(stdout,"adaptive sampling stopping criteria reached: %d matches out of %d in total sampled and stabilized\n",t.missing_matches_suspect,i);
            break;
          }
        
          /*
          if (i > 20000 && p.adaptive_matches_stop > 0 && p.adaptive_matches_stop < t.missing_matches_suspect) {
            fprintf(stdout,"adaptive sampling stopping criteria reached: %d matches out of %d in total sampled\n",t.missing_matches_suspect,i);
            break;
          }
          */
        }
        #endif
      }
      else fprintf(stderr,"."); 
      fflush(stderr);
    }
    if (i%p.picgap==0) {
      get_counter_filename(picname,picstr,pic_count++);
      printalltreeinfo(picname,&t,&p);
    }
  }
#ifdef FORENSIC
  FILE *OUTFOR;
  print_forensic(stdout,&match);
  OUTFOR=openstemleafoutputfile(picstr,"forensic");
  print_forensic(OUTFOR,&match); 
  printhaplist(stdout,&match);
  printhaplist(OUTFOR,&match);
  fclose(OUTFOR);
#endif
 
  strcpy(picname,picstr);strcat(picname,".end");
  printalltreeinfo(picname,&t,&p);   
  rescale_proportions(&t,p.reps,p.treebetN,p.Nbetsamp);
  print_proportions(stdout,&t);
  print_proportions(PARS,&t);
  fprintf(stdout,"missing matches suspect: %d\n",t.missing_matches_suspect);
  fprintf(stdout,"missing doesn't match suspect: %d\n",t.missing_doesnt_match_suspect);
  
  /*
  if (t.missing_matches_suspect + t.missing_doesnt_match_suspect != p.reps*p.Nbetsamp) {
    fprintf(stdout,"ERROR: missing_matches_suspect + missing_doesnt_match_suspect != reps * Nbetsamp\n");
    exit(1);
  }
  */
  
  match_prob_est = (double)t.missing_matches_suspect / (double)(p.reps*p.Nbetsamp);
  match_prob_ci = 1.959964 * sqrt( (match_prob_est * (1 - match_prob_est)) / (double)(p.reps*p.Nbetsamp) );
  fprintf(stdout,"Binomial match probability: %f%% (Wald CI: [%f%% ; %f%%])\n", 100*match_prob_est, 100*(match_prob_est - match_prob_ci), 100*(match_prob_est + match_prob_ci));
  fprintf(stdout,"LR = 1 / match probability: %f (using Wald CI: [%f ; %f])\n", 1/match_prob_est, 1/(match_prob_est + match_prob_ci), 1/(match_prob_est - match_prob_ci));
  if (p.tconsensus) fclose(CONSENSUST);
  if (p.pconsensus) fclose(CONSENSUSP);

  time(&end_time);
  time_diff = difftime(end_time, start_time);
  OUTTIME=openstemleafoutputfile(picstr,"time");
  fprintf(OUTTIME,"%.2lf",time_diff);
  fclose(OUTTIME);

  fclose(OUTFILE);
  fclose(PARS);
  destroy_tree(&t);
  return 1;
}
/**************************************************************************/


